const Instantiated = require("./instantiated_node.js");
const fs = require("fs");
let zlib = require('zlib');
let args=process.argv;
let writer = null;

let sessionid="";
let passkey="";
let inviteCode="";
let listOnly=false;
let file=[];
let fileCounter=0;

for(var i=1; i<args.length; i++) {
    switch(args[i]) {
        case "--session":
            sessionid=args[i+1];
            break;
        case "--password":
            passkey=args[i+1];
            break;
        case "--code":
            inviteCode=args[i+1];
            break;
        case "--listOnly":
            listOnly=true
            break;
        case "--file":
            if(listOnly == false) {
                file=args[i+1].split(",");
            }
            break;
        case "--help":
            console.log("List of commands available for the client.");
            console.log("--session [Session ID] = Sets the session id for joining.");
            console.log("--password [Password] = Sets the password incase the session requires a password.");
            console.log("--code [Invite Code] = Sets the invite code.");
            console.log("--listOnly = Sets the list only to true. After listing close the program immediately.");
            console.log("--file [Filename1.png,Filename2.png] = If listOnly is set to false, download the files then exit.");
            console.log("--help = Displays the list of commands for clients.");
            process.exit();
            break;
        default:
            console.log("Unknown command. Try using --help to get a list of commands.");
            break;
    }
}

let instance = new Instantiated();

let States={
    fileSize: -1,
    fileDownloaded: -1
}

instance.onOpen(() => {
    instance.SetSessionID(sessionid);
    instance.JoinServer(sessionid, passkey, inviteCode);
    setInterval(function() {
        instance.Ping();
    }, 15000);
    
});

instance.onMessage((data) => {
    var id=JSON.parse(data.toString("utf8"));
    // console.log(id);

    switch(id["MSG"]) {
        case "SocketID":
            instance.SetSocketID(id["SocketID"]);
            break;
        case "Secure_Rx":
            var srx=id["DataStream"];

            switch(srx["id"]) {
                case "directory_list":
                    console.clear();
                    for(var i=0; i<srx["directory"].length; i++) {
                        console.log(srx["directory"][i] + " - " + srx["sizes"][i] + " bytes");
                    }

                    if(listOnly==false) {
                        instance.Send({
                            id: "download_file",
                            filename: file[fileCounter]
                        });
                    } else {
                        process.exit(1);
                    }
                    break;
                case "begin_download":
                    console.log("Downloading "+srx["filename"]+" at "+srx["size"]+" bytes...");
                    States.fileSize = srx["size"];
                    States.fileDownloaded = 0;

                    writer=fs.createWriteStream(srx["filename"]);
                    break;
                case "downloaded_bytes":
                    writer.write(zlib.inflateRawSync(Buffer.from(srx["bytes"])));
                    States.fileDownloaded+=srx["bytes"]["data"].length;
                    console.clear();
                    console.log("Downloaded "+(States.fileDownloaded / States.fileSize * 100) + "%");
                    break;
                case "finished_writing":
                    // writer.close();
                    console.clear();
                    States.fileDownloaded = 0;
                    fileCounter++;
                    if(fileCounter >= file.length) {
                        process.exit(1);
                    } else {
                        instance.Send({
                            id: "download_file",
                            filename: file[fileCounter]
                        });
                    }
                    break;
            }
            break;
        case "ServerDisconnected":
            console.log("The host has disconnected from the session. Goodbye!");
            process.exit(0);
            break;
        case "session_not_found":
            console.log("Session not found!");
            process.exit(0);
    }
})