const Instantiated = require("./instantiated_node.js");
const fs = require("fs");
let zlib = require('zlib');

let config=null;

if(fs.existsSync("settings.json")) {
    config = JSON.parse(fs.readFileSync("settings.json"));
} else {
    config = {
        session_id: "MyFiles",
        password: "password",
        whitelist: false,
        createInvite: false,
        max_connections: 50,
        sent_per_bytes: 1024 * 6,
        rootdir: "uploaded/",
        banned: [],
        allowed: [],
        allow_uploading: false
    }
    fs.writeFileSync("settings.json", JSON.stringify(config, null, '\t'));
}

let instance = new Instantiated();
instance.onOpen(() => {
    if(config["session_id"]=="") {
        instance.CreateSession("FileShare", config["max_connections"], config["whitelist"], config["password"], null, false, false, true);
    } else {
        instance.CreateServer(config["session_id"], "FileShare", config["max_connections"], config["whitelist"], config["password"], false);
    }

    setInterval(function() {
        instance.Ping();
    }, 15000);
});

let clients = {};

instance.onMessage((data) => {
    var id=JSON.parse(data.toString("utf8"));
    // console.log(id);

    switch(id["MSG"]) {
        case "SocketID":
            instance.SetSocketID(id["SocketID"]);
            break;
        case "created_session":
            instance.SetSessionID(id["SessionGUID"]);

            if(config["createInvite"] == true) {
                instance.CreateInvite(50);
            }

            for(var i=0; i<config["banned"].length; i++) {
                instance.BanIP(config["banned"][i], null, null, null);
            }

            for(var i=0; i<config["allowed"].length; i++) {
                instance.AddIPToWhitelist(config["allowed"][i]);
            }

            console.log("Session ID: "+instance.GetSessionID()+" has been created!");
            break;
        case "Invite_Created":
            console.log("Invite code created: "+id["Code"]+" with a limited max use of "+id["Limits"]+".");
            break;
        case "joined_session":
            console.log("Client #"+id["SocketID"]+" has joined the session.");
            var d=fs.readdirSync(config["rootdir"]);
            var sl=[];
            for(var is=0; is<d.length; is++) {
                var check=fs.statSync(config["rootdir"]+d[is]);
                sl.push(check["size"]);
            }

            instance.PrivateSend({
                id: "directory_list",
                directory: d,
                sizes: sl
            }, id["SocketID"]);

            clients[id["SocketID"]] = {
                file_descriptor: null
            };
            break;
        case "Tx":
            var tx=id["DataStream"];

            switch(tx["id"]) {
                case "download_file":
                    console.log("Client #"+id["SocketID"]+" is requesting to download "+tx["filename"]+".");
                    console.log("The request came from ["+id["IPAddress"]+"]");
                    var file=config["rootdir"] + tx["filename"];
                    var check=fs.statSync(file);
                    instance.PrivateSend({
                        id: "begin_download",
                        size: check["size"],
                        filename: tx["filename"]
                    }, id["SocketID"]);

                    clients[id["SocketID"]]["file_descriptor"]=fs.createReadStream(file, {highWaterMark: config["sent_per_bytes"]});

                    clients[id["SocketID"]]["file_descriptor"].on("data", (chunk) => {
                        var dataset = Buffer.from(chunk)
                        instance.PrivateSend({
                            id: "downloaded_bytes",
                            bytes: zlib.deflateRawSync(dataset)
                        }, id["SocketID"]);
                    });

                    clients[id["SocketID"]]["file_descriptor"].on("end", () => {
                        instance.PrivateSend({
                            id: "finished_writing",
                            size: check["size"],
                            filename: tx["filename"]
                        }, id["SocketID"]);
                        clients[id["SocketID"]]["file_descriptor"].close();
                    });
                    break;
                case "begin_upload":
                    console.log("Uploading is under work in progress at the moment.");
                    break;
            }
            break;
        case "Disconnected":
            console.log("Client #"+id["SocketID"]+" has lefted the session.");

            delete clients[id["SocketID"]];
            break;
    }
})